@@ -982,50 +982,19 @@ pub fn get_stmt_trailing_trivia(stmt: Stmt) -> (Stmt, Vec<Token>) {
     (updated_stmt, trailing_trivia)
 }
 
-pub fn get_last_stmt_trailing_trivia(last_stmt: LastStmt) -> (LastStmt, Vec<Token>) {
+pub fn last_stmt_trailing_trivia(last_stmt: &LastStmt) -> Vec<Token> {
     match last_stmt {
         LastStmt::Return(ret) => {
-            let mut return_token = ret.token().to_owned();
-            let mut formatted_expression_list = ret.returns().to_owned();
-            let mut trailing_trivia = Vec::new();
-
-            // Retrieve last item and remove trailing trivia
-            if let Some(last_pair) = formatted_expression_list.pop() {
-                let pair = last_pair.map(|value| {
-                    trailing_trivia = get_expression_trailing_trivia(&value);
-                    value.update_trailing_trivia(FormatTriviaType::Replace(vec![]))
-                });
-                formatted_expression_list.push(pair);
+            if ret.returns().is_empty() {
+                ret.token().trailing_trivia().cloned().collect()
             } else {
-                trailing_trivia = return_token
-                    .trailing_trivia()
-                    .map(|x| x.to_owned())
-                    .collect();
-                return_token =
-                    return_token.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+                let last_expression = ret.returns().iter().last().unwrap();
+                get_expression_trailing_trivia(last_expression)
             }
-
-            (
-                LastStmt::Return(
-                    ret.with_token(return_token)
-                        .with_returns(formatted_expression_list),
-                ),
-                trailing_trivia,
-            )
-        }
-        LastStmt::Break(token) => {
-            let trailing_trivia = token.trailing_trivia().map(|x| x.to_owned()).collect();
-            let token = token.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
-
-            (LastStmt::Break(token), trailing_trivia)
         }
+        LastStmt::Break(token) => token.trailing_trivia().cloned().collect(),
         #[cfg(feature = "luau")]
-        LastStmt::Continue(token) => {
-            let trailing_trivia = token.trailing_trivia().map(|x| x.to_owned()).collect();
-            let token = token.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
-
-            (LastStmt::Continue(token), trailing_trivia)
-        }
+        LastStmt::Continue(token) => token.trailing_trivia().cloned().collect(),
         other => panic!("unknown node {:?}", other),
     }
 }
